@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 #include "draw-imp.h"
 
@@ -45,20 +67,18 @@
 	special case code in the edgebuffer based rasterizer to cope with this.
 */
 
-
-
 static void
-line(fz_context *ctx, fz_rasterizer *rast, const fz_matrix *ctm, float x0, float y0, float x1, float y1)
+line(fz_context *ctx, fz_rasterizer *rast, fz_matrix ctm, float x0, float y0, float x1, float y1)
 {
-	float tx0 = ctm->a * x0 + ctm->c * y0 + ctm->e;
-	float ty0 = ctm->b * x0 + ctm->d * y0 + ctm->f;
-	float tx1 = ctm->a * x1 + ctm->c * y1 + ctm->e;
-	float ty1 = ctm->b * x1 + ctm->d * y1 + ctm->f;
+	float tx0 = ctm.a * x0 + ctm.c * y0 + ctm.e;
+	float ty0 = ctm.b * x0 + ctm.d * y0 + ctm.f;
+	float tx1 = ctm.a * x1 + ctm.c * y1 + ctm.e;
+	float ty1 = ctm.b * x1 + ctm.d * y1 + ctm.f;
 	fz_insert_rasterizer(ctx, rast, tx0, ty0, tx1, ty1, 0);
 }
 
 static void
-bezier(fz_context *ctx, fz_rasterizer *rast, const fz_matrix *ctm, float flatness,
+bezier(fz_context *ctx, fz_rasterizer *rast, fz_matrix ctm, float flatness,
 	float xa, float ya,
 	float xb, float yb,
 	float xc, float yc,
@@ -112,7 +132,7 @@ bezier(fz_context *ctx, fz_rasterizer *rast, const fz_matrix *ctm, float flatnes
 }
 
 static void
-quad(fz_context *ctx, fz_rasterizer *rast, const fz_matrix *ctm, float flatness,
+quad(fz_context *ctx, fz_rasterizer *rast, fz_matrix ctm, float flatness,
 	float xa, float ya,
 	float xb, float yb,
 	float xc, float yc, int depth)
@@ -153,7 +173,7 @@ quad(fz_context *ctx, fz_rasterizer *rast, const fz_matrix *ctm, float flatness,
 typedef struct
 {
 	fz_rasterizer *rast;
-	const fz_matrix *ctm;
+	fz_matrix ctm;
 	float flatness;
 	fz_point b;
 	fz_point c;
@@ -218,7 +238,7 @@ static void
 flatten_rectto(fz_context *ctx, void *arg_, float x0, float y0, float x1, float y1)
 {
 	flatten_arg *arg = (flatten_arg *)arg_;
-	const fz_matrix *ctm = arg->ctm;
+	fz_matrix ctm = arg->ctm;
 
 	flatten_moveto(ctx, arg_, x0, y0);
 
@@ -226,21 +246,21 @@ flatten_rectto(fz_context *ctx, void *arg_, float x0, float y0, float x1, float
 	{
 		/* In the case where we have an axis aligned rectangle, do some
 		 * horrid antidropout stuff. */
-		if (ctm->b == 0 && ctm->c == 0)
+		if (ctm.b == 0 && ctm.c == 0)
 		{
-			float tx0 = ctm->a * x0 + ctm->e;
-			float ty0 = ctm->d * y0 + ctm->f;
-			float tx1 = ctm->a * x1 + ctm->e;
-			float ty1 = ctm->d * y1 + ctm->f;
+			float tx0 = ctm.a * x0 + ctm.e;
+			float ty0 = ctm.d * y0 + ctm.f;
+			float tx1 = ctm.a * x1 + ctm.e;
+			float ty1 = ctm.d * y1 + ctm.f;
 			fz_insert_rasterizer_rect(ctx, arg->rast, tx0, ty0, tx1, ty1);
 			return;
 		}
-		else if (ctm->a == 0 && ctm->d == 0)
+		else if (ctm.a == 0 && ctm.d == 0)
 		{
-			float tx0 = ctm->c * y0 + ctm->e;
-			float ty0 = ctm->b * x0 + ctm->f;
-			float tx1 = ctm->c * y1 + ctm->e;
-			float ty1 = ctm->b * x1 + ctm->f;
+			float tx0 = ctm.c * y0 + ctm.e;
+			float ty0 = ctm.b * x0 + ctm.f;
+			float tx1 = ctm.c * y1 + ctm.e;
+			float ty1 = ctm.b * x1 + ctm.f;
 			fz_insert_rasterizer_rect(ctx, arg->rast, tx0, ty1, tx1, ty0);
 			return;
 		}
@@ -264,27 +284,10 @@ static const fz_path_walker flatten_proc =
 	flatten_rectto
 };
 
-int
-fz_flatten_fill_path(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, const fz_matrix *ctm, float flatness, const fz_irect *scissor, fz_irect *bbox)
+static int
+do_flatten_fill(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, fz_matrix ctm, float flatness)
 {
 	flatten_arg arg;
-	fz_irect local_bbox;
-
-	if (fz_reset_rasterizer(ctx, rast, scissor))
-	{
-		arg.rast = rast;
-		arg.ctm = ctm;
-		arg.flatness = flatness;
-		arg.b.x = arg.b.y = arg.c.x = arg.c.y = 0;
-
-		fz_walk_path(ctx, path, &flatten_proc, &arg);
-		if (arg.c.x != arg.b.x || arg.c.y != arg.b.y)
-			line(ctx, rast, ctm, arg.c.x, arg.c.y, arg.b.x, arg.b.y);
-
-		fz_gap_rasterizer(ctx, rast);
-
-		fz_postindex_rasterizer(ctx, rast);
-	}
 
 	arg.rast = rast;
 	arg.ctm = ctm;
@@ -297,24 +300,42 @@ fz_flatten_fill_path(fz_context *ctx, fz_rasterizer *rast, const fz_path *path,
 
 	fz_gap_rasterizer(ctx, rast);
 
-	if (!bbox)
-		return 0;
-
-	local_bbox = *scissor;
-	fz_bound_rasterizer(ctx, rast, bbox);
-	return fz_is_empty_irect(fz_intersect_irect(bbox, &local_bbox));
+	return fz_is_empty_irect(fz_bound_rasterizer(ctx, rast));
 }
 
-enum {
-	ONLY_MOVES = 0,
-	NON_NULL_LINE = 1,
-	NULL_LINE
-};
+int
+fz_flatten_fill_path(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, fz_matrix ctm, float flatness, fz_irect scissor, fz_irect *bbox)
+{
+	int empty;
+	fz_irect local_bbox;
+	if (!bbox)
+		bbox = &local_bbox;
+
+	/* If we're given an empty scissor, sanitize it. This makes life easier
+	 * down the line. */
+	if (fz_is_empty_irect(scissor))
+		scissor.x1 = scissor.x0, scissor.y1 = scissor.y0;
+
+	if (fz_reset_rasterizer(ctx, rast, scissor))
+	{
+		empty = do_flatten_fill(ctx, rast, path, ctm, flatness);
+		if (empty)
+			return *bbox = fz_empty_irect, 1;
+		fz_postindex_rasterizer(ctx, rast);
+	}
+
+	empty = do_flatten_fill(ctx, rast, path, ctm, flatness);
+	if (empty)
+		return *bbox = fz_empty_irect, 1;
+
+	*bbox = fz_intersect_irect(scissor, fz_bound_rasterizer(ctx, rast));
+	return fz_is_empty_irect(*bbox);
+}
 
 typedef struct sctx
 {
 	fz_rasterizer *rast;
-	const fz_matrix *ctm;
+	fz_matrix ctm;
 	float flatness;
 	const fz_stroke_state *stroke;
 
@@ -324,7 +345,7 @@ typedef struct sctx
 	fz_point beg[2];
 	fz_point seg[2];
 	int sn;
-	int dot;
+	int not_just_moves;
 	int from_bezier;
 	fz_point cur;
 
@@ -338,15 +359,18 @@ typedef struct sctx
 	float phase;
 	fz_point dash_cur;
 	fz_point dash_beg;
+
+	float dirn_x;
+	float dirn_y;
 } sctx;
 
 static void
 fz_add_line(fz_context *ctx, sctx *s, float x0, float y0, float x1, float y1, int rev)
 {
-	float tx0 = s->ctm->a * x0 + s->ctm->c * y0 + s->ctm->e;
-	float ty0 = s->ctm->b * x0 + s->ctm->d * y0 + s->ctm->f;
-	float tx1 = s->ctm->a * x1 + s->ctm->c * y1 + s->ctm->e;
-	float ty1 = s->ctm->b * x1 + s->ctm->d * y1 + s->ctm->f;
+	float tx0 = s->ctm.a * x0 + s->ctm.c * y0 + s->ctm.e;
+	float ty0 = s->ctm.b * x0 + s->ctm.d * y0 + s->ctm.f;
+	float tx1 = s->ctm.a * x1 + s->ctm.c * y1 + s->ctm.e;
+	float ty1 = s->ctm.b * x1 + s->ctm.d * y1 + s->ctm.f;
 
 	fz_insert_rasterizer(ctx, s->rast, tx0, ty0, tx1, ty1, rev);
 }
@@ -355,21 +379,21 @@ static void
 fz_add_horiz_rect(fz_context *ctx, sctx *s, float x0, float y0, float x1, float y1)
 {
 	if (fz_antidropout_rasterizer(ctx, s->rast)) {
-		if (s->ctm->b == 0 && s->ctm->c == 0)
+		if (s->ctm.b == 0 && s->ctm.c == 0)
 		{
-			float tx0 = s->ctm->a * x0 + s->ctm->e;
-			float ty0 = s->ctm->d * y0 + s->ctm->f;
-			float tx1 = s->ctm->a * x1 + s->ctm->e;
-			float ty1 = s->ctm->d * y1 + s->ctm->f;
+			float tx0 = s->ctm.a * x0 + s->ctm.e;
+			float ty0 = s->ctm.d * y0 + s->ctm.f;
+			float tx1 = s->ctm.a * x1 + s->ctm.e;
+			float ty1 = s->ctm.d * y1 + s->ctm.f;
 			fz_insert_rasterizer_rect(ctx, s->rast, tx1, ty1, tx0, ty0);
 			return;
 		}
-		else if (s->ctm->a == 0 && s->ctm->d == 0)
+		else if (s->ctm.a == 0 && s->ctm.d == 0)
 		{
-			float tx0 = s->ctm->c * y0 + s->ctm->e;
-			float ty0 = s->ctm->b * x0 + s->ctm->f;
-			float tx1 = s->ctm->c * y1 + s->ctm->e;
-			float ty1 = s->ctm->b * x1 + s->ctm->f;
+			float tx0 = s->ctm.c * y0 + s->ctm.e;
+			float ty0 = s->ctm.b * x0 + s->ctm.f;
+			float tx1 = s->ctm.c * y1 + s->ctm.e;
+			float ty1 = s->ctm.b * x1 + s->ctm.f;
 			fz_insert_rasterizer_rect(ctx, s->rast, tx1, ty0, tx0, ty1);
 			return;
 		}
@@ -384,21 +408,21 @@ fz_add_vert_rect(fz_context *ctx, sctx *s, float x0, float y0, float x1, float y
 {
 	if (fz_antidropout_rasterizer(ctx, s->rast))
 	{
-		if (s->ctm->b == 0 && s->ctm->c == 0)
+		if (s->ctm.b == 0 && s->ctm.c == 0)
 		{
-			float tx0 = s->ctm->a * x0 + s->ctm->e;
-			float ty0 = s->ctm->d * y0 + s->ctm->f;
-			float tx1 = s->ctm->a * x1 + s->ctm->e;
-			float ty1 = s->ctm->d * y1 + s->ctm->f;
+			float tx0 = s->ctm.a * x0 + s->ctm.e;
+			float ty0 = s->ctm.d * y0 + s->ctm.f;
+			float tx1 = s->ctm.a * x1 + s->ctm.e;
+			float ty1 = s->ctm.d * y1 + s->ctm.f;
 			fz_insert_rasterizer_rect(ctx, s->rast, tx0, ty1, tx1, ty0);
 			return;
 		}
-		else if (s->ctm->a == 0 && s->ctm->d == 0)
+		else if (s->ctm.a == 0 && s->ctm.d == 0)
 		{
-			float tx0 = s->ctm->c * y0 + s->ctm->e;
-			float ty0 = s->ctm->b * x0 + s->ctm->f;
-			float tx1 = s->ctm->c * y1 + s->ctm->e;
-			float ty1 = s->ctm->b * x1 + s->ctm->f;
+			float tx0 = s->ctm.c * y0 + s->ctm.e;
+			float ty0 = s->ctm.b * x0 + s->ctm.f;
+			float tx1 = s->ctm.c * y1 + s->ctm.e;
+			float ty1 = s->ctm.b * x1 + s->ctm.f;
 			fz_insert_rasterizer_rect(ctx, s->rast, tx0, ty0, tx1, ty1);
 			return;
 		}
@@ -648,18 +672,11 @@ fz_add_line_join(fz_context *ctx, sctx *s, float ax, float ay, float bx, float b
 }
 
 static void
-fz_add_line_cap(fz_context *ctx, sctx *s, float ax, float ay, float bx, float by, fz_linecap linecap, int rev)
+do_linecap(fz_context *ctx, sctx *s, float bx, float by, fz_linecap linecap, int rev, float dlx, float dly)
 {
 	float flatness = s->flatness;
 	float linewidth = s->linewidth;
 
-	float dx = bx - ax;
-	float dy = by - ay;
-
-	float scale = linewidth / sqrtf(dx * dx + dy * dy);
-	float dlx = dy * scale;
-	float dly = -dx * scale;
-
 	switch (linecap)
 	{
 	case FZ_LINECAP_BUTT:
@@ -710,6 +727,38 @@ fz_add_line_cap(fz_context *ctx, sctx *s, float ax, float ay, float bx, float by
 	}
 }
 
+static void
+fz_add_line_cap(fz_context *ctx, sctx *s, float ax, float ay, float bx, float by, fz_linecap linecap, int rev)
+{
+	float linewidth = s->linewidth;
+	float dx = bx - ax;
+	float dy = by - ay;
+
+	float scale = linewidth / sqrtf(dx * dx + dy * dy);
+	float dlx = dy * scale;
+	float dly = -dx * scale;
+
+	do_linecap(ctx, s, bx, by, linecap, rev, dlx, dly);
+}
+
+static void
+fz_add_zero_len_cap(fz_context *ctx, sctx *s, float ax, float ay, fz_linecap linecap, int rev)
+{
+	float linewidth = s->linewidth;
+	float dx = rev ? -s->dirn_x : s->dirn_x;
+	float dy = rev ? -s->dirn_y : s->dirn_y;
+	float scale, dlx, dly;
+
+	if (dx == 0 && dy == 0)
+		return;
+
+	scale = linewidth / sqrtf(dx * dx + dy * dy);
+	dlx = dy * scale;
+	dly = -dx * scale;
+
+	do_linecap(ctx, s, ax, ay, linecap, rev, dlx, dly);
+}
+
 static void
 fz_add_line_dot(fz_context *ctx, sctx *s, float ax, float ay)
 {
@@ -740,13 +789,24 @@ fz_add_line_dot(fz_context *ctx, sctx *s, float ax, float ay)
 static void
 fz_stroke_flush(fz_context *ctx, sctx *s, fz_linecap start_cap, fz_linecap end_cap)
 {
-	if (s->sn == 2)
+	if (s->sn == 1)
 	{
 		fz_add_line_cap(ctx, s, s->beg[1].x, s->beg[1].y, s->beg[0].x, s->beg[0].y, start_cap, 2);
 		fz_add_line_cap(ctx, s, s->seg[0].x, s->seg[0].y, s->seg[1].x, s->seg[1].y, end_cap, 0);
 	}
-	else if (s->dot == NULL_LINE)
-		fz_add_line_dot(ctx, s, s->beg[0].x, s->beg[0].y);
+	else if (s->not_just_moves)
+	{
+		if (s->cap == FZ_LINECAP_ROUND)
+		{
+			fz_add_line_dot(ctx, s, s->beg[0].x, s->beg[0].y);
+		}
+		else
+		{
+			fz_add_zero_len_cap(ctx, s, s->beg[0].x, s->beg[0].y, s->cap, 2);
+			fz_add_zero_len_cap(ctx, s, s->beg[0].x, s->beg[0].y, s->cap, 0);
+		}
+	}
+
 	fz_gap_rasterizer(ctx, s->rast);
 }
 
@@ -757,29 +817,36 @@ fz_stroke_moveto(fz_context *ctx, void *s_, float x, float y)
 
 	s->seg[0].x = s->beg[0].x = x;
 	s->seg[0].y = s->beg[0].y = y;
-	s->sn = 1;
-	s->dot = ONLY_MOVES;
+	s->sn = 0;
+	s->not_just_moves = 0;
 	s->from_bezier = 0;
+	s->dirn_x = 0;
+	s->dirn_y = 0;
 }
 
 static void
-fz_stroke_lineto(fz_context *ctx, sctx *s, float x, float y, int from_bezier)
+fz_stroke_lineto_aux(fz_context *ctx, sctx *s, float x, float y, int from_bezier, float dirn_x, float dirn_y)
 {
-	float ox = s->seg[s->sn-1].x;
-	float oy = s->seg[s->sn-1].y;
+	float ox = s->seg[s->sn].x;
+	float oy = s->seg[s->sn].y;
 	float dx = x - ox;
 	float dy = y - oy;
 	float dlx, dly;
 
+	s->not_just_moves = 1;
+
+	/* We store the direction (as used for the alignment of caps etc) based on the
+	 * direction we are passed in. */
+	s->dirn_x = dirn_x;
+	s->dirn_y = dirn_y;
+
+	/* We calculate the normal vectors from the delta that we have just moved. */
 	if (find_normal_vectors(dx, dy, s->linewidth, &dlx, &dly))
 	{
-		if (s->dot == ONLY_MOVES && (s->cap == FZ_LINECAP_ROUND || s->dash_list))
-			s->dot = NULL_LINE;
 		return;
 	}
-	s->dot = NON_NULL_LINE;
 
-	if (s->sn == 2)
+	if (s->sn == 1)
 		fz_add_line_join(ctx, s, s->seg[0].x, s->seg[0].y, ox, oy, x, y, s->from_bezier & from_bezier);
 
 #if 1
@@ -799,7 +866,7 @@ fz_stroke_lineto(fz_context *ctx, sctx *s, float x, float y, int from_bezier)
 		fz_add_line(ctx, s, x + dlx, y + dly, ox + dlx, oy + dly, 1);
 	}
 
-	if (s->sn == 2)
+	if (s->sn)
 	{
 		s->seg[0] = s->seg[1];
 		s->seg[1].x = x;
@@ -809,39 +876,50 @@ fz_stroke_lineto(fz_context *ctx, sctx *s, float x, float y, int from_bezier)
 	{
 		s->seg[1].x = s->beg[1].x = x;
 		s->seg[1].y = s->beg[1].y = y;
-		s->sn = 2;
+		s->sn = 1;
 	}
 	s->from_bezier = from_bezier;
 }
 
+static void
+fz_stroke_lineto(fz_context *ctx, sctx *s, float x, float y, int from_bezier)
+{
+	float ox = s->seg[s->sn].x;
+	float oy = s->seg[s->sn].y;
+	float dx = x - ox;
+	float dy = y - oy;
+	fz_stroke_lineto_aux(ctx, s, x, y, from_bezier, dx, dy);
+}
+
 static void
 fz_stroke_closepath(fz_context *ctx, sctx *s)
 {
-	if (s->sn == 2)
+	if (s->sn == 1)
 	{
 		fz_stroke_lineto(ctx, s, s->beg[0].x, s->beg[0].y, 0);
 		/* fz_stroke_lineto will *normally* end up with s->seg[1] being the x,y coords passed in.
 		 * As such, the following line should draw a linejoin between the closing segment of this
 		 * subpath (seg[0]->seg[1]) == (seg[0]->beg[0]) and the first segment of this subpath
 		 * (beg[0]->beg[1]).
-		 * In cases where the line was already at an x,y infinitessimally close to s->beg[0],
+		 * In cases where the line was already at an x,y infinitesimally close to s->beg[0],
 		 * fz_stroke_lineto may exit without doing any processing. This leaves seg[0]->seg[1]
 		 * pointing at the penultimate line segment. Thus this draws a linejoin between that
 		 * penultimate segment and the end segment. This is what we want. */
 		fz_add_line_join(ctx, s, s->seg[0].x, s->seg[0].y, s->beg[0].x, s->beg[0].y, s->beg[1].x, s->beg[1].y, 0);
 	}
-	else if (s->dot == NULL_LINE)
+	else if (s->not_just_moves && s->cap == FZ_LINECAP_ROUND)
 		fz_add_line_dot(ctx, s, s->beg[0].x, s->beg[0].y);
 
 	s->seg[0] = s->beg[0];
-	s->sn = 1;
-	s->dot = ONLY_MOVES;
+	s->sn = 0;
+	s->not_just_moves = 0;
 	s->from_bezier = 0;
+	s->dirn_x = 0;
+	s->dirn_y = 0;
 
 	fz_gap_rasterizer(ctx, s->rast);
 }
 
-
 static void
 fz_stroke_bezier(fz_context *ctx, struct sctx *s,
 	float xa, float ya,
@@ -1197,7 +1275,7 @@ b_moved_vertically:	/* d and dy have the same sign */
 
 		if (s->toggle)
 		{
-			fz_stroke_lineto(ctx, s, mx, my, from_bezier);
+			fz_stroke_lineto_aux(ctx, s, mx, my, from_bezier, dx, dy);
 		}
 		else
 		{
@@ -1222,7 +1300,7 @@ b_moved_vertically:	/* d and dy have the same sign */
 
 		if (s->toggle)
 		{
-			fz_stroke_lineto(ctx, s, bx, by, from_bezier);
+			fz_stroke_lineto_aux(ctx, s, bx, by, from_bezier, dx, dy);
 		}
 	}
 	else
@@ -1233,7 +1311,7 @@ adjust_for_tail:
 		/* Update the position in the dash array */
 		if (s->toggle)
 		{
-			fz_stroke_lineto(ctx, s, old_bx, old_by, from_bezier);
+			fz_stroke_lineto_aux(ctx, s, old_bx, old_by, from_bezier, dx, dy);
 		}
 		else
 		{
@@ -1256,7 +1334,7 @@ adjust_for_tail:
 		}
 		if (s->toggle)
 		{
-			fz_stroke_lineto(ctx, s, old_bx, old_by, from_bezier);
+			fz_stroke_lineto_aux(ctx, s, old_bx, old_by, from_bezier, dx, dy);
 		}
 		else
 		{
@@ -1421,7 +1499,7 @@ static const fz_path_walker dash_proc =
 };
 
 static int
-do_flatten_stroke(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, const fz_stroke_state *stroke, const fz_matrix *ctm, float flatness, float linewidth, const fz_irect *scissor, fz_irect *bbox)
+do_flatten_stroke(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, const fz_stroke_state *stroke, fz_matrix ctm, float flatness, float linewidth)
 {
 	struct sctx s;
 	const fz_path_walker *proc = &stroke_proc;
@@ -1434,10 +1512,12 @@ do_flatten_stroke(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, con
 	s.linewidth = linewidth * 0.5f; /* hairlines use a different value from the path value */
 	s.miterlimit = stroke->miterlimit;
 	s.sn = 0;
-	s.dot = ONLY_MOVES;
+	s.not_just_moves = 0;
 	s.toggle = 0;
 	s.offset = 0;
 	s.phase = 0;
+	s.dirn_x = 0;
+	s.dirn_y = 0;
 
 	s.cap = stroke->start_cap;
 
@@ -1456,10 +1536,10 @@ do_flatten_stroke(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, con
 		if (s.dash_total == 0)
 			return 1;
 
-		fz_scissor_rasterizer(ctx, rast, &s.rect);
+		s.rect = fz_scissor_rasterizer(ctx, rast);
 		if (fz_try_invert_matrix(&inv, ctm))
 			return 1;
-		fz_transform_rect(&s.rect, &inv);
+		s.rect = fz_transform_rect(s.rect, inv);
 		s.rect.x0 -= linewidth;
 		s.rect.x1 += linewidth;
 		s.rect.y0 -= linewidth;
@@ -1478,23 +1558,29 @@ do_flatten_stroke(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, con
 	fz_walk_path(ctx, path, proc, &s);
 	fz_stroke_flush(ctx, &s, s.cap, stroke->end_cap);
 
-	if (!bbox)
-		return 0;
-
-	return fz_is_empty_irect(fz_bound_rasterizer(ctx, rast, bbox));
+	return fz_is_empty_irect(fz_bound_rasterizer(ctx, rast));
 }
 
 int
-fz_flatten_stroke_path(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, const fz_stroke_state *stroke, const fz_matrix *ctm, float flatness, float linewidth, const fz_irect *scissor, fz_irect *bbox)
+fz_flatten_stroke_path(fz_context *ctx, fz_rasterizer *rast, const fz_path *path, const fz_stroke_state *stroke, fz_matrix ctm, float flatness, float linewidth, fz_irect scissor, fz_irect *bbox)
 {
+	int empty;
+	fz_irect local_bbox;
+	if (!bbox)
+		bbox = &local_bbox;
 
 	if (fz_reset_rasterizer(ctx, rast, scissor))
 	{
-		if (do_flatten_stroke(ctx, rast, path, stroke, ctm, flatness, linewidth, scissor, bbox))
-			return 1;
+		empty = do_flatten_stroke(ctx, rast, path, stroke, ctm, flatness, linewidth);
+		if (empty)
+			return *bbox = fz_empty_irect, 1;
 		fz_postindex_rasterizer(ctx, rast);
-		bbox = NULL;
 	}
 
-	return do_flatten_stroke(ctx, rast, path, stroke, ctm, flatness, linewidth, scissor, bbox);
+	empty = do_flatten_stroke(ctx, rast, path, stroke, ctm, flatness, linewidth);
+	if (empty)
+		return *bbox = fz_empty_irect, 1;
+
+	*bbox = fz_intersect_irect(scissor, fz_bound_rasterizer(ctx, rast));
+	return fz_is_empty_irect(*bbox);
 }
